# Pyrubrum - An intuitive framework for creating Telegram bots
# Copyright (C) 2020 Hearot <https://github.com/hearot>
#
# This file is part of Pyrubrum.
#
# Pyrubrum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyrubrum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyrubrum. If not, see <https://www.gnu.org/licenses/>.

from typing import Optional
from typing import Union

from pyrogram import InputMedia
from pyrogram.client.filters.filters import Filter

from pyrubrum.menus.styles import BaseStyle
from pyrubrum.menus.styles import PageStyle
from pyrubrum.types import Types
from .menu import Menu


class PageMenu(Menu):
    """Implementation of a menu which automatically, given a list of items, manages
    paging and the setting of parameters. It has got, by definition, all the
    parameters defined in `Menu` plus a list of items that will be displayed
    to the user.

    See Also:
        For understanding how the keyboard is being generated, see `PageStyle`.

    Parameters:
        name (str): The name you give to the menu, which will be used as
            the text of callback button, if needed. See `BaseMenu` for more
            information.
        menu_id (str): The unique identifier given to the menu, which will
            refer unequivocally to this entity. The hash for this class is
            generated relying on the content of this field. Avoid using ``0``
            as it is used for buttons whose purpose is only related to design
            (i.e. they do not point to any menu, see :term:`Null-pointer
            button`). See `BaseMenu` for more information.
        content (Types.Content): What will be displayed whenever a user
            accesses this menu. Both text and media can be provided. A
            function can be provided as well and must follow the following
            arguments pattern::

                func(handler, client, context, parameters)

            See `Menu` for more information.
        items (types.Items): The list of elements the menu is compounded of or
            a function which returns such type of value.
        deep_link (Optional[bool]): If this menu shall be reached by a
            deep-link whose payload is the identifier of this instance.
            Defaults to ``False``.
        default (Optional[bool]): If this menu shall be displayed if no
            other :term:`top-level menu <Top-level menu>` has been matched.
            It works only if this menu is a :term:`top-level <Top-level menu>`
            one.
        message_filter (Optional[Filter]): A filter for telling Pyrogram
            when a message should be associated to this menu. It works only
            for top-level menus (see `Handler.setup`). Defaults to ``None``,
            which automatically makes this menu reachable when the user texts
            a message that follows this pattern::

                /[MENU_ID]

        preliminary (Types.Preliminary): A function which is executed each time
            before doing anything else in `on_callback` and `on_message`.
            You can provide a list of such functions as well, which will be
            executed following the same order as the one of the list.
            Defaults to ``None``, which means that no function is going to
            be executed.
        style (BaseStyle): The class which generates the keyboard for this
            function following a certain style. Defaults to `PageStyle()`.

    Warning:
        This implementation is not compatible with a non-parameterized handler.
        An handler that supports parameterization is required.
    """

    def __init__(
        self,
        name: str,
        menu_id: str,
        content: Union[InputMedia, str],
        items: Types.Items,
        deep_link: Optional[bool] = False,
        default: Optional[bool] = False,
        message_filter: Optional[Filter] = None,
        preliminary: Types.Preliminary = None,
        style: BaseStyle = PageStyle(),
    ):
        """Roughly equivalent to::

            Menu(self, name, menu_id, content, default, message_filter,
                 preliminary, style, items=items)
        """
        Menu.__init__(
            self,
            name,
            menu_id,
            content,
            deep_link=deep_link,
            default=default,
            message_filter=message_filter,
            preliminary=preliminary,
            style=style,
        )

        self.items = items



# Pyrubrum - An intuitive framework for creating Telegram bots
# Copyright (C) 2020 Hearot <https://github.com/hearot>
#
# This file is part of Pyrubrum.
#
# Pyrubrum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyrubrum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyrubrum. If not, see <https://www.gnu.org/licenses/>.

from typing import List
from typing import Set
from typing import Union

from environs import Env
from pyrogram import Client

from pyrubrum import DeepLinkMenu
from pyrubrum import DictDatabase
from pyrubrum import Element
from pyrubrum import Menu
from pyrubrum import Node
from pyrubrum import PageMenu
from pyrubrum import ParameterizedHandler
from pyrubrum import RedisDatabase
from pyrubrum import transform

try:
    from redis import Redis
except (ImportError, ModuleNotFoundError):
    pass

drinks = [
    "Beer",
    "Cider",
    "Coffee",
    "Coke",
    "French wine",
    "Gin",
    "Hard soda",
    "Irish beer",
    "Italian wine",
    "Liquor",
    "Water",
    "Wine",
]
snacks = [
    "Bubblegum",
    "Candies",
    "Chocolate bar",
    "Crisps",
    "Cracker nuts",
    "Crêpe",
    "Pancake",
    "Pizza",
    "Pretzel",
    "Waffle",
]


def make_elements(elements: List[str]) -> List[Element]:
    return [Element(name, index) for index, name in enumerate(elements)]


tree = transform(
    {
        Menu(
            "Start",
            "start",
            "ℹ️ Have a drink or a snack by saying /drink or /snack",
            default=True,
        ): [
            DeepLinkMenu("🍷 Drink", "link_drink", "drink"),
            DeepLinkMenu("🍬 Snack", "link_snack", "snack"),
        ],
        PageMenu(
            "Drink",
            "drink",
            "🍷 Choose the drink you want!",
            make_elements(drinks),
            deep_link=True,
        ): {Menu("After drink choice", "drink_chosen", "🍹 Here you go!")},
        PageMenu(
            "Snack",
            "snack",
            "🍬 Choose the snack you want!",
            make_elements(snacks),
            deep_link=True,
        ): {Menu("After snack choice", "snack_chosen", "🥨 Here you go!")},
    }
)


def main(
    api_hash: str,
    api_id: int,
    bot_token: str,
    database: Union[DictDatabase, RedisDatabase],
    session_name: str,
    tree: Set[Node],
):
    bot = Client(
        session_name, api_hash=api_hash, api_id=api_id, bot_token=bot_token
    )

    handler = ParameterizedHandler(tree, database)
    handler.setup(bot)

    bot.run()


if __name__ == "__main__":
    env = Env()
    env.read_env()

    api_hash = env("API_HASH")
    api_id = env.int("API_ID")
    bot_token = env("BOT_TOKEN")
    session_name = env("SESSION_NAME")

    if env.bool("USE_REDIS", False):
        db = env.int("REDIS_DB")
        host = env("REDIS_HOST")
        port = env.int("REDIS_PORT")
        database = RedisDatabase(Redis(host=host, port=port, db=db))
    else:
        database = DictDatabase()

    main(api_hash, api_id, bot_token, database, session_name, tree)



# Pyrubrum - An intuitive framework for creating Telegram bots
# Copyright (C) 2020 Hearot <https://github.com/hearot>
#
# This file is part of Pyrubrum.
#
# Pyrubrum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyrubrum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyrubrum. If not, see <https://www.gnu.org/licenses/>.

from typing import Any
from typing import Callable
from typing import Dict
from typing import Optional
from typing import Union

from pyrogram import CallbackQuery
from pyrogram import Client
from pyrogram import InlineKeyboardMarkup
from pyrogram import InputMedia
from pyrogram import Message
from pyrogram.client.filters.filters import Filter

from pyrubrum.keyboard import Keyboard
from pyrubrum.menus.styles import MenuStyle
from pyrubrum.menus.styles import BaseStyle
from pyrubrum.types import Types
from .base_menu import BaseMenu


class Menu(BaseMenu):
    """Implementation of a menu who has got, by definition, a content to display
    (i.e. what the user will see by accessing it), a name and a style.

    See Also:
        For complete examples of styles you can make use of:

        * `MenuStyle`
        * `PageStyle` (although it is recommended to use it only
          with `PageMenu`)

    Parameters:
        name (str): The name you give to the menu, which will be used as
            the text of callback button, if needed. See `BaseMenu` for more
            information.
        menu_id (str): The unique identifier given to the menu, which will
            refer unequivocally to this entity. The hash for this class is
            generated relying on the content of this field. Avoid using ``0``
            as it is used for buttons whose purpose is only related to
            design (i.e. they do not point to any menu,
            see :term:`Null-pointer button`). See `BaseMenu` for more
            information.
        content (Types.Content): What will be displayed whenever a user
            accesses this menu. Both text and media can be provided. A
            function that returns such values can be provided as well
            and must follow the this arguments pattern::

                func(handler, client, context, parameters)

        deep_link (Optional[bool]): If this menu shall be reached by a
            deep-link whose payload is the identifier of this instance.
            Defaults to ``False``.
        default (Optional[bool]): If this menu shall be displayed if no
            other :term:`top-level menu <Top-level menu>` has been matched.
            It works only if this menu is a :term:`top-level <Top-level menu>`
            one.
        message_filter (Optional[Filter]): A filter for telling Pyrogram
            when a message should be associated to this menu. It works only
            for :term:`top-level menus <Top-level menu>` (see `Handler.setup`).
            Defaults to ``None``, which automatically makes this menu reachable
            when the user texts a message that follows this pattern::

                /[MENU_ID]

        preliminary (Types.Preliminary): A function which is executed each time
            before doing anything else in `on_callback` and `on_message`.
            You can provide a list of such functions as well, which will be
            executed following the same order as the one of the list.
            Defaults to ``None``, which means that no function is going to
            be executed.
        style (BaseStyle): The class which generates the keyboard for this
            function following a certain style. Defaults to `MenuStyle()`.

    Note:
        This implementation is compatible with a non-parameterized handler.
    """

    def __init__(
        self,
        name: str,
        menu_id: str,
        content: Union[
            Union[InputMedia, str],
            Callable[
                [
                    "Handler",
                    Client,
                    Union[CallbackQuery, Message],
                    Dict[str, Any],
                ],
                Union[InputMedia, str],
            ],
        ],
        deep_link: Optional[bool] = False,
        default: Optional[bool] = False,
        message_filter: Optional[Filter] = None,
        preliminary: Types.Preliminary = None,
        style: BaseStyle = MenuStyle(),
        **kwargs
    ):
        BaseMenu.__init__(self, name, menu_id)
        self.content = content
        self.deep_link = deep_link
        self.default = default
        self.message_filter = message_filter
        self.preliminary = preliminary
        self.style = style

        for argument in kwargs:
            if not hasattr(self, argument):
                setattr(self, argument, kwargs[argument])

    def get_content(
        self,
        handler: "Handler",  # noqa
        client: Client,
        context: Union[CallbackQuery, Message],
        parameters: Optional[Dict[str, Any]] = None,
    ) -> Union[InputMedia, str]:
        """Get the content which will be displayed to the user.

        Parameters:
            handler (BaseHandler): The handler which coordinates the management
                of the menus.
            client (Client): The client which is linked to the handler.
            context (Union[CallbackQuery, Message]): The context for which the
                button is generated.
            parameters (Optional[Dict[str, Any]]): The parameters which were
                passed to the handler. Defaults to ``None``.

        Returns:
            Union[InputMedia, str]: The content of the menu, which is then
            displayed to the user as a media (if it is a subclass of
            `pyrogram.InputMedia`) or a message (if it is just a string).
        """
        if callable(self.content):
            if isinstance(parameters, dict):
                return self.content(handler, client, context, parameters)
            else:
                return self.content(handler, client, context)

        return self.content

    def on_callback(
        self,
        handler: "Handler",  # noqa
        client: Client,
        callback: CallbackQuery,
        parameters: Optional[Dict[str, Any]] = None,
    ):
        """Each time a callback query is handled, this function calls the preliminary
        function (i.e. `Menu.preliminary`), if callable, then gets the content
        that is going to be provided to the user (both text and media are
        compatible), sets up an inline keyboard using the style defined in
        `Menu.style` and finally edits the message with
        `CallbackQuery.edit_message_text` (if the content is a string, i.e. a
        text) or `CallbackQuery.edit_message_media`
        (if the content is an instance of `pyrogram.InputMedia`, i.e. a media).

        Parameters:
            handler (BaseHandler): The handler which coordinates the management
                of the menus.
            client (Client): The client which is linked to the handler.
            context (CallbackQuery): The callback query for which the button is
                generated.
            parameters (Optional[Dict[str, Any]]): The parameters which were
                passed to the handler. Defaults to ``None``.
        """

        if callable(self.preliminary):
            self.preliminary(handler, client, callback, parameters)
        elif isinstance(self.preliminary, list):
            for func in self.preliminary:
                func(handler, client, callback, parameters)

        content = self.get_content(handler, client, callback, parameters)

        if isinstance(content, InputMedia):
            callback.edit_message_media(
                content,
                reply_markup=self.keyboard(
                    handler, client, callback, parameters
                ),
            )
        elif isinstance(content, str):
            callback.edit_message_text(
                content,
                reply_markup=self.keyboard(
                    handler, client, callback, parameters
                ),
            )
        else:
            raise TypeError("content must be of type InputMedia or str")

    def on_message(
        self,
        handler: "Handler",  # noqa
        client: Client,
        message: Message,
        parameters: Optional[Dict[str, Any]] = None,
    ):
        """Each time a message is handled, this function calls the preliminary
        function (i.e. `Menu.preliminary`), if callable, then gets the content
        that is going to be provided to the user (both text and media are
        compatible), sets up an inline keyboard using the style defined in
        `Menu.style` and finally sends the message with `Message.reply_text`
        (if the content is a string, i.e. a text) or
        `Message.reply_cached_media` (if the content is an instance of
        `pyrogram.InputMedia`, i.e. a media).

        Parameters:
            handler (BaseHandler): The handler which coordinates the management
                of the menus.
            client (Client): The client which is linked to the handler.
            context (Message): The message for which the button is generated.
            parameters (Optional[Dict[str, Any]]): The parameters which were
                passed to the handler. Defaults to ``None``.
        """

        if callable(self.preliminary):
            self.preliminary(handler, client, message, parameters)
        elif isinstance(self.preliminary, list):
            for func in self.preliminary:
                func(handler, client, message, parameters)

        content = self.get_content(handler, client, message, parameters)

        if isinstance(content, InputMedia):
            message.reply_cached_media(
                file_id=content.media,
                file_ref=content.file_ref,
                caption=content.caption,
                reply_markup=self.keyboard(
                    handler, client, message, parameters
                ),
            )
        elif isinstance(content, str):
            message.reply_text(
                content,
                reply_markup=self.keyboard(
                    handler, client, message, parameters
                ),
            )
        else:
            raise TypeError("content must be of type InputMedia or str")

    def keyboard(
        self,
        handler: "Handler",  # noqa
        client: Client,
        context: Union[CallbackQuery, Message],
        parameters: Optional[Dict[str, Any]] = None,
    ) -> InlineKeyboardMarkup:
        """Provide a keyboard using the chosen style in `Menu.style`.

        Parameters:
            handler (BaseHandler): The handler which coordinates the management
                of the menus.
            client (Client): The client which is linked to the handler.
            context (Union[CallbackQuery, Message]): The context for which the
                button is generated.
            parameters (Dict[str, Any]): The parameters which were passed to
                the handler.

        Returns:
            pyrogram.InlineKeyboardMarkup: The generated inline keyboard,
            which is then displayed to the user.
        """

        keyboard = self.style.generate(
            handler, client, context, parameters, self
        )

        if isinstance(context, Message):
            return (
                Keyboard(
                    keyboard,
                    handler,
                    str(context.message_id) + str(context.from_user.id),
                    context.chat.id,
                )
                if keyboard
                else None
            )
        elif isinstance(context, CallbackQuery):
            return (
                Keyboard(
                    keyboard, handler, context.id, context.message.chat.id
                )
                if keyboard
                else None
            )



# Pyrubrum - An intuitive framework for creating Telegram bots
# Copyright (C) 2020 Hearot <https://github.com/hearot>
#
# This file is part of Pyrubrum.
#
# Pyrubrum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyrubrum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyrubrum. If not, see <https://www.gnu.org/licenses/>.

from datetime import timedelta
from typing import Any
from typing import Callable
from typing import Dict
from typing import Optional
from typing import List
from typing import Union

from pyrogram import Client
from pyrogram import InputMedia

from pyrubrum.keyboard.element import Element


class Types:
    Callback = Callable[
        ["BaseHandler", Client, Any, Optional[Dict[str, Any]]], None
    ]
    """This type defines the possible values for functions that work as
    callbacks in Pyrubrum (e.g. `on_callback`, `on_message`). They must
    implement the following pattern to be valid::

        callback(handler, client, context, parameters=None)
    """

    Content = Union[
        Union[InputMedia, str],
        Callable[
            ["Handler", Client, Any, Dict[str, Any]], Union[InputMedia, str],
        ],
    ]
    """This type defines the possible values for the content that is provided as
    argument to a menu. A string or an instance of `InputMedia
    <pyrogram.InputMedia>` is suitable for this type to be valid.
    A function that returns such values is valid as well and must follow this
    pattern::

        content(handler, client, context, parameters=None)
    """

    Expire = Optional[Union[bool, int, timedelta]]
    """This type defines the possible values for an expire (see `BaseDatabase`).
    A positive integer is suitable and indicates how many seconds a value can
    be kept within database. A `datetime.timedelta` can be provided and is
    valid as well. In order to indicate that there is no expire, ``False``
    shall be provided.
    """

    Items = Union[
        List[Element],
        Callable[
            ["ParameterizedHandler", Client, Any, Dict[str, Any]],
            List[Element],
        ],
    ]
    """This type defines the possible values for items that are provided as
    argument to `PageMenu`. A list of `Element` is suitable for this type.
    A function that returns such list is valid as well and must follow this
    pattern::


        items(handler, client, context, parameters)
    """

    Link = Union[
        str, Callable[["Handler", Client, Any, Dict[str, Any]], str],
    ]
    """This type defines the possible values for the link that is used to build
    an inline button that redirects to a website. A string is suitable for this
    type to be valid.
    A function that returns such value is valid as well and must follow this
    pattern::

        link(handler, client, context, parameters=None)
    """

    Payload = Union[
        str, Callable[["Handler", Client, Any, Dict[str, Any]], str],
    ]
    """This type defines the possible values for the parameter that is passed
    to the bot using a deep-link. A string is suitable for this type to be
    valid.
    A function that returns such value is valid as well and must follow this
    pattern::

        payload(handler, client, context, parameters=None)
    """

    Preliminary = Optional[
        Union[
            Callable[["Handler", Client, Any, Dict[str, Any]], None],
            List[Callable[["Handler", Client, Any, Dict[str, Any]], None]],
        ]
    ]
    """This type defines the possible values which can be passed for a
    preliminary function. All the provided functions for this type must
    follow this pattern::

        preliminary(handler, client, context, parameters=None)

    A list of such functions can be provided as well for this type to
    be valid.
    """

    PyrogramCallback = Callable[[Client, Any], None]
    """This type describes the pattern of functions which are added to
    a Pyrogram handler, being it::

        callback(client, context)
    """



# Pyrubrum - An intuitive framework for creating Telegram bots
# Copyright (C) 2020 Hearot <https://github.com/hearot>
#
# This file is part of Pyrubrum.
#
# Pyrubrum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyrubrum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyrubrum. If not, see <https://www.gnu.org/licenses/>.

from packaging import version

import pyrubrum


def test_type():
    assert isinstance(pyrubrum.__version__, str)


def test_pattern():
    version.Version(pyrubrum.__version__)



# Pyrubrum - An intuitive framework for creating Telegram bots
# Copyright (C) 2020 Hearot <https://github.com/hearot>
#
# This file is part of Pyrubrum.
#
# Pyrubrum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyrubrum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyrubrum. If not, see <https://www.gnu.org/licenses/>.

from calendar import monthrange
from datetime import datetime
from typing import Set
from typing import Union

from environs import Env
from pyrogram import Client

from pyrubrum import DictDatabase
from pyrubrum import Element
from pyrubrum import Menu
from pyrubrum import Node
from pyrubrum import PageMenu
from pyrubrum import PageStyle
from pyrubrum import ParameterizedHandler
from pyrubrum import RedisDatabase
from pyrubrum import transform

try:
    from redis import Redis
except (ImportError, ModuleNotFoundError):
    pass


def generate_days(handler, client, context, parameters):
    month = int(parameters["month_id"])
    year = int(parameters["year_id"])

    return [
        Element(str(day + 1), str(day + 1))
        for day in range(monthrange(year, month)[1])
    ]


def generate_months():
    return [Element(str(month + 1), str(month + 1)) for month in range(12)]


def generate_years(start: int, end: int):
    return [Element(str(year), str(year)) for year in range(start, end + 1)]


def tell_about_the_day(handler, client, context, parameters):
    day = int(parameters["day_id"])
    month = int(parameters["month_id"])
    year = int(parameters["year_id"])

    return "📅 " + datetime(year, month, day).strftime("%d/%m/%Y is a %A.")


tree = transform(
    {
        PageMenu(
            "Start",
            "start",
            "📅 Choose a year.",
            generate_years(1970, 2044),
            default=True,
            style=PageStyle(limit=4, limit_items=15),
        ): {
            PageMenu(
                "After year choice",
                "year",
                "📅 Choose a month.",
                generate_months(),
                style=PageStyle(limit=5, limit_items=12),
            ): {
                PageMenu(
                    "After month choice",
                    "month",
                    "📅 Choose a day.",
                    generate_days,
                    style=PageStyle(limit=5, limit_items=31),
                ): {Menu("After day choice", "day", tell_about_the_day)}
            }
        }
    }
)


def main(
    api_hash: str,
    api_id: int,
    bot_token: str,
    database: Union[DictDatabase, RedisDatabase],
    session_name: str,
    tree: Set[Node],
):
    bot = Client(
        session_name, api_hash=api_hash, api_id=api_id, bot_token=bot_token
    )

    handler = ParameterizedHandler(tree, database)
    handler.setup(bot)

    bot.run()


if __name__ == "__main__":
    env = Env()
    env.read_env()

    api_hash = env("API_HASH")
    api_id = env.int("API_ID")
    bot_token = env("BOT_TOKEN")
    session_name = env("SESSION_NAME")

    if env.bool("USE_REDIS", False):
        db = env.int("REDIS_DB")
        host = env("REDIS_HOST")
        port = env.int("REDIS_PORT")
        database = RedisDatabase(Redis(host=host, port=port, db=db))
    else:
        database = DictDatabase()

    main(api_hash, api_id, bot_token, database, session_name, tree)
